{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "f6f51d13-60e2-478c-8e11-1dd61e1cc330",
   "metadata": {},
   "source": [
    "# Plotly ボタンウィジェット\n",
    "\n",
    "公開日 <time datetime=\"2025-01-05\">2025-01-05</time>\n",
    "\n",
    "Pythonのインタラクティブなグラフを描画できるライブラリPlotlyにて、ボタンウィジェットを追加する方法を解説します。\n",
    "\n",
    "Plotlyには、高水準のAPIであるPlotly Expressと呼ばれるものもありますが、この記事では細かい調節ができる`graph_objects`を対象としています。"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "b6e8c83e-4884-490a-a85b-352a61150a08",
   "metadata": {},
   "source": [
    "<!-- START MoshimoAffiliateEasyLink -->\n",
    "<script type=\"text/javascript\">\n",
    "(function(b,c,f,g,a,d,e){b.MoshimoAffiliateObject=a;\n",
    "b[a]=b[a]||function(){arguments.currentScript=c.currentScript\n",
    "||c.scripts[c.scripts.length-2];(b[a].q=b[a].q||[]).push(arguments)};\n",
    "c.getElementById(a)||(d=c.createElement(f),d.src=g,\n",
    "d.id=a,e=c.getElementsByTagName(\"body\")[0],e.appendChild(d))})\n",
    "(window,document,\"script\",\"//dn.msmstatic.com/site/cardlink/bundle.js?20220329\",\"msmaflink\");\n",
    "msmaflink({\"n\":\"pandas\\u0026Plotly 2D\\/3Dデータビジュアライゼーション実装ハンドブック\",\"b\":\"秀和システム\",\"t\":\"\",\"d\":\"https:\\/\\/m.media-amazon.com\",\"c_p\":\"\\/images\\/I\",\"p\":[\"\\/51buhQYpkrL._SL500_.jpg\",\"\\/41vnATKu5RL._SL500_.jpg\",\"\\/41vDFPPpE1L._SL500_.jpg\",\"\\/4150EmIK-VL._SL500_.jpg\",\"\\/41-G1+HL-lL._SL500_.jpg\",\"\\/41fwd4F77vL._SL500_.jpg\",\"\\/31LL+GnIWuL._SL500_.jpg\",\"\\/41XR6ul2enL._SL500_.jpg\",\"\\/41RCmqHCXuL._SL500_.jpg\",\"\\/41qtNs0Fe-L._SL500_.jpg\"],\"u\":{\"u\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"t\":\"amazon\",\"r_v\":\"\"},\"v\":\"2.1\",\"b_l\":[{\"id\":1,\"u_tx\":\"Amazonで見る\",\"u_bc\":\"#f79256\",\"u_url\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"a_id\":4672318,\"p_id\":170,\"pl_id\":27060,\"pc_id\":185,\"s_n\":\"amazon\",\"u_so\":1},{\"id\":2,\"u_tx\":\"楽天市場で見る\",\"u_bc\":\"#f76956\",\"u_url\":\"https:\\/\\/search.rakuten.co.jp\\/search\\/mall\\/pandas%26Plotly%202D%2F3D%E3%83%87%E3%83%BC%E3%82%BF%E3%83%93%E3%82%B8%E3%83%A5%E3%82%A2%E3%83%A9%E3%82%A4%E3%82%BC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E5%AE%9F%E8%A3%85%E3%83%8F%E3%83%B3%E3%83%89%E3%83%96%E3%83%83%E3%82%AF\\/\",\"a_id\":4672316,\"p_id\":54,\"pl_id\":27059,\"pc_id\":54,\"s_n\":\"rakuten\",\"u_so\":2}],\"eid\":\"IXitR\",\"s\":\"l\"});\n",
    "</script>\n",
    "<div id=\"msmaflink-IXitR\">リンク</div>\n",
    "<!-- MoshimoAffiliateEasyLink END -->"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cbab4bd3-bb15-40c1-b463-ffb8d4454a64",
   "metadata": {},
   "source": [
    "## ボタンウィジェットの基礎\n",
    "\n",
    "Plotlyのヒートマップ `go.Heatmap` に、カラーマップを切り替えるボタンを追加する例を示します。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "63a0db3c-425f-400c-bf85-714831d48642",
   "metadata": {},
   "outputs": [],
   "source": [
    "import plotly.graph_objects as go\n",
    "\n",
    "z_data = [\n",
    "    [1, 2, 3],\n",
    "    [4, 5, 6],\n",
    "    [7, 8, 9],\n",
    "]\n",
    "\n",
    "fig = go.Figure(\n",
    "    data=go.Heatmap(z=z_data, colorscale=\"Viridis\")\n",
    ")\n",
    "\n",
    "buttons = [\n",
    "    dict(\n",
    "        args=[\"colorscale\", \"Viridis\"],\n",
    "        label=\"Viridis\",\n",
    "        method=\"restyle\",\n",
    "    ),\n",
    "    dict(\n",
    "        args=[\"colorscale\", \"Jet\"],\n",
    "        label=\"Jet\",\n",
    "        method=\"restyle\",\n",
    "    ),\n",
    "    dict(\n",
    "        args=[\"colorscale\", \"Reds\"],\n",
    "        label=\"Reds\",\n",
    "        method=\"restyle\",\n",
    "    ),\n",
    "]\n",
    "\n",
    "fig.update_layout(\n",
    "    updatemenus=[\n",
    "        dict(\n",
    "            type=\"buttons\",\n",
    "            direction=\"left\",\n",
    "            buttons=buttons,\n",
    "            pad={\"r\": 10, \"t\": 10},\n",
    "            showactive=True,\n",
    "            x=0.05,\n",
    "            xanchor=\"left\",\n",
    "            y=1.3,\n",
    "            yanchor=\"top\",\n",
    "        ),\n",
    "    ]\n",
    ")\n",
    "\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2d3a776a-edb7-46ae-b58c-9f4811ff3430",
   "metadata": {},
   "source": [
    "上記の例を解説します。\n",
    "\n",
    "まず、各ボタンを以下のように辞書で定義します。\n",
    "\n",
    "```python\n",
    "dict(\n",
    "    args=[\"colorscale\", \"Viridis\"],\n",
    "    label=\"Viridis\",\n",
    "    method=\"restyle\",\n",
    "),\n",
    "```\n",
    "\n",
    "各キーの意味は以下の通りです。\n",
    "\n",
    "- `args`: ボタンで設定したいパラメータ\n",
    "- `label`: ボタンに表示するラベル名\n",
    "- `method`: `\"restyle\"`（データを変更）、`\"relayout\"`（レイアウトを変更）、`\"update\"`（レイアウトとデータの両方を変更）"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cac92260-253d-4bca-bb25-e88eeb620a06",
   "metadata": {},
   "source": [
    "次に、`fig.update_layout()`メソッドの`updatemenus`引数でボタンを表示します。\n",
    "\n",
    "```python\n",
    "fig.update_layout(\n",
    "    updatemenus=[\n",
    "        dict(\n",
    "            type=\"buttons\",\n",
    "            direction=\"left\",\n",
    "            buttons=buttons,\n",
    "            pad={\"r\": 10, \"t\": 10},\n",
    "            showactive=True,\n",
    "            x=0.05,\n",
    "            xanchor=\"left\",\n",
    "            y=1.3,\n",
    "            yanchor=\"top\",\n",
    "        ),\n",
    "    ]\n",
    ")\n",
    "```\n",
    "\n",
    "各キーの意味は以下の通りです。\n",
    "\n",
    "- `type` (str): メニューの種類。`\"buttons\"`, `\"dropdown\"`\n",
    "- `direction` (str): ボタンを配置する方向。`\"left\"`（横方向）、`\"top\"`（縦方向）\n",
    "- `buttons` (list): 各ボタンの定義\n",
    "- `pad` (dict): ボタン周りの空白 (pixel).\n",
    "- `showactive` (bool): `True`の場合、選択したボタンを目立たせる\n",
    "- `x` (float): x方向の位置（0: 左端。1: 右端）\n",
    "- `y` (float): y方向の位置（0: 下端。1: 上端）\n",
    "\n",
    "また、上記以外に`updatemenus`に指定可能なオプションを以下に示します。\n",
    "\n",
    "- `active` (int): 初期状態でアクティブにするボタン（0始まりの整数）\n",
    "- `bgcolor` (str): 非アクティブ状態のボタンの背景色\n",
    "- `bordercolor` (str): ボタンの枠線の色\n",
    "- `borderwidth` (int): ボタンの枠線の太さ（デフォルト値：`1`）\n",
    "\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "1e4a021f-5b6a-4cc0-b3fd-c1eedb96cd41",
   "metadata": {},
   "source": [
    "## relayoutによるタイトル表示\n",
    "\n",
    "続いて、`method=\"relayout\"`としてレイアウトを変更する例を示します。\n",
    "ボタンによって、図のタイトルの表示・非表示を切り替えます。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "1b10dc3b-9943-4b7e-b8c6-3dec002701f7",
   "metadata": {},
   "outputs": [],
   "source": [
    "fig = go.Figure()\n",
    "fig.add_trace(go.Scatter(x=[1, 2, 3], y=[1, 3, 2], name=\"Series A\"))\n",
    "fig.add_trace(go.Scatter(x=[1, 2, 3], y=[3, 2, 1], name=\"Series B\"))\n",
    "fig.update_layout(title=\"Figure Title\")\n",
    "\n",
    "buttons = [\n",
    "    dict(\n",
    "        args=[\"title\", \"Figure Title\"],\n",
    "        label=\"Show title\",\n",
    "        method=\"relayout\",\n",
    "    ),\n",
    "    dict(\n",
    "        args=[\"title\",  \"\"],\n",
    "        label=\"Hide title\",\n",
    "        method=\"relayout\",\n",
    "    ),\n",
    "]\n",
    "\n",
    "fig.update_layout(\n",
    "    updatemenus=[\n",
    "        dict(\n",
    "            type=\"buttons\",\n",
    "            direction=\"left\",\n",
    "            buttons=buttons,\n",
    "            pad={\"r\": 10, \"t\": 10},\n",
    "            showactive=True,\n",
    "            x=0.2,\n",
    "            xanchor=\"left\",\n",
    "            y=1.3,\n",
    "            yanchor=\"top\",\n",
    "        ),\n",
    "    ]\n",
    ")\n",
    "\n",
    "fig.show()"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
